# -*- coding: utf-8 -*-
# :Project:   hurm -- SQLAlchemy declaration of table activitypayloads
# :Created:   sab 27 feb 2016 16:40:06 CET
# :Author:    Lele Gaifax <lele@metapensiero.it>
# :License:   GNU General Public License version 3 or later
# :Copyright: © 2016 Lele Gaifax
#

import sqlalchemy as sa

from .. import metadata, translatable_string as _
from . import domains


activitypayloads = sa.Table(
    'activitypayloads', metadata,

    sa.Column(
        'idactivitypayload', domains.integer_t, sa.Sequence('gen_idactivitypayload',
                                                            optional=True),
        nullable=False,
        primary_key=True,
        info={'label': _("Activity payload ID"),
              'hint': _("Unique ID of the activity payload")}
    ),

    sa.Column(
        'idactivity', domains.integer_t, sa.ForeignKey('activities.idactivity'),
        nullable=False,
        info={'label': _("Activity ID"),
              'hint': _("ID of the related activity")}),

    sa.Column(
        'description', domains.description_t,
        nullable=False,
        info={'label': _("Value name"),
              'hint': _("The name of the particular payload value")},
    ),

    sa.Column(
        'note', domains.text_t,
        info={'label': _("Note"),
              'hint': _("The long description of the payload value")}),

    sa.Column(
        'unitcost', domains.money_t,
        info={'label': _("Unit cost"),
              'hint': _("Coefficient to transform the value in money")}),
)



# -*- coding: utf-8 -*-
# :Project:   hurm -- Duty class, mapped to table duties
# :Created:   sab 02 gen 2016 15:33:23 CET
# :Author:    Lele Gaifax <lele@metapensiero.it>
# :License:   GNU General Public License version 3 or later
# :Copyright: © 2016 Lele Gaifax
#

from . import AbstractBase


class Duty(AbstractBase):
    def __init__(self, **kwargs):
        payloads = kwargs.pop('payloads', None)
        if payloads:
            self._update_payloads(payloads)
        super().__init__(**kwargs)

    def __str__(self):
        return '%s, %s' % (self.person, self.task)

    def update(self, data, **kwargs):
        payloads = data.pop('payloads', None)
        if payloads:
            changes = self._update_payloads(payloads)
        else:
            changes = {}

        changes.update(super().update(data, **kwargs))

        return changes

    def _update_payloads(self, payloads):
        from .dutypayload import DutyPayload

        changes = {}

        for payload in payloads:
            for existing in self.payloads:
                if existing.idactivitypayload == payload['idactivitypayload']:
                    changes[existing.activity_payload.description] = (
                        existing.value, payload['value'])
                    existing.value = payload['value']
                    break
            else:
                self.payloads.append(
                    DutyPayload(idactivitypayload=payload['idactivitypayload'],
                                value=payload['value']))

        return changes



# -*- coding: utf-8 -*-
# :Project:   hurm -- Tests on Person entity
# :Created:   gio 03 mar 2016 11:08:57 CET
# :Author:    Lele Gaifax <lele@metapensiero.it>
# :License:   GNU General Public License version 3 or later
# :Copyright: © 2016 Lele Gaifax
#

from hurm.db.entities import Person


def test_repr(jane_tree):
    assert "Tree Jane" in repr(jane_tree)


def test_preferred_activities(session, subtitles, supervision):
    foobar = Person(firstname='Foo', lastname='Bar',
                    idactivities='%d,%d' % (subtitles.idactivity,
                                            supervision.idactivity))
    session.add(foobar)
    session.commit()

    assert len(foobar.activities) == 2

    foobar.update(dict(idactivities=''))
    session.commit()

    assert len(foobar.activities) == 0

    foobar.delete(session)
    session.commit()


def test_deletion_with_preferred_activities(session, subtitles, supervision):
    foobar = Person(firstname='Foo', lastname='Bar',
                    idactivities='%d,%d' % (subtitles.idactivity,
                                            supervision.idactivity))
    session.add(foobar)
    session.commit()

    assert len(foobar.activities) == 2

    foobar.delete(session)
    try:
        session.commit()
    finally:
        session.rollback()
